Package: PRONE
Type: Package
Title: The PROteomics Normalization Evaluator
Version: 1.7.0
Authors@R: c(person("Lis", "Arend", email = "lis.arend@tum.de", role = c("aut", "cre"), comment = c(ORCID = "0000-0001-7990-8385")))
Description: High-throughput omics data are often affected by systematic biases introduced throughout all the steps of a clinical study, from sample collection to quantification. 
  Normalization methods aim to adjust for these biases to make the actual biological signal more prominent. 
  However, selecting an appropriate normalization method is challenging due to the wide range of available approaches. 
  Therefore, a comparative evaluation of unnormalized and normalized data is essential in identifying an appropriate normalization strategy for a specific data set. 
  This R package provides different functions for preprocessing, normalizing, and evaluating different normalization approaches. 
  Furthermore, normalization methods can be evaluated on downstream steps, such as differential expression analysis and statistical enrichment analysis. 
  Spike-in data sets with known ground truth and real-world data sets of biological experiments acquired by either tandem mass tag (TMT) or label-free quantification (LFQ) can be analyzed.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.2
Depends: R (>= 4.4.0), SummarizedExperiment
Imports: dplyr, magrittr, data.table, RColorBrewer, ggplot2, S4Vectors,
        ComplexHeatmap, stringr, NormalyzerDE, tibble, limma, MASS,
        edgeR, matrixStats, preprocessCore, stats, gtools, methods,
        ROTS, ComplexUpset, tidyr, purrr, circlize, gprofiler2,
        plotROC, MSnbase, UpSetR, dendsort, vsn, Biobase, reshape2,
        POMA, ggtext, scales, DEqMS, vegan
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, BiocStyle, DT
BugReports: https://github.com/daisybio/PRONE/issues
URL: https://github.com/daisybio/PRONE
VignetteBuilder: knitr
Config/testthat/edition: 3
biocViews: Proteomics, Preprocessing, Normalization,
        DifferentialExpression, Visualization
git_url: https://git.bioconductor.org/packages/PRONE
git_branch: devel
git_last_commit: cee8ec3
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-07-14
NeedsCompilation: no
Packaged: 2026-07-14 23:38:40 UTC; biocbuild
Author: Lis Arend [aut, cre] (ORCID: <https://orcid.org/0000-0001-7990-8385>)
Maintainer: Lis Arend <lis.arend@tum.de>
